# encoding: utf-8

# ------------------------------------------------------------------------------
# (C) Copyright 2019 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# ------------------------------------------------------------------------------
# Copyright (c) 2006 Novell, Inc. All Rights Reserved.
#
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of version 2 of the GNU General Public License as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail, you may find
# current contact information at www.novell.com.
# ------------------------------------------------------------------------------

# File:	include/novell-schematool/dialogs.ycp
# Package:	Configuration of novell-schematool
# Summary:	Dialogs definitions
# Authors:	Mark Clark <mclark@novell.com>
#
# $Id: dialogs.ycp 27914 2006-02-13 14:32:08Z locilka $
module Yast
  module NovellSchematoolDialogsInclude
    def initialize_novell_schematool_dialogs(include_target)
      Yast.import "UI"

      textdomain "novell-schematool"

      Yast.import "Label"
      Yast.import "NovellLDAP"
      Yast.import "NovellLDAPUI"
      Yast.import "NovellSchematool"
      Yast.import "NovellUtils"
      Yast.import "NovellValidators"
      Yast.import "Wizard"

      Yast.include include_target, "novell-schematool/complex.rb"
      Yast.include include_target, "novell-schematool/helps.rb"

      # globals
      @services_list = [] # oes services patterns
      @checked_items = [] # items that have been checked in the MultiSelection box.
    end

    # schematool LDAP Dialog
    #
    # @return dialog result
    def LDAPDialog
      # schematool LDAP dialog caption
      caption = _("OES eDirectory Extension Utility")

      ldap_server = Ops.get_string(
        NovellSchematool.Settings,
        "CONFIG_SCHEMATOOL_LDAP_SERVER",
        ""
      )
      admin_name = Ops.get_string(
        NovellSchematool.Settings,
        "CONFIG_SCHEMATOOL_LDAP_ADMIN",
        ""
      )
      admin_password = Ops.get_string(
        NovellSchematool.Settings,
        "CONFIG_SCHEMATOOL_LDAP_ADMIN_PASSWORD",
        ""
      )
      ldap_port = Ops.get_integer(
        NovellSchematool.Settings,
        "CONFIG_SCHEMATOOL_LDAP_SECURE_PORT",
        636
      )

      contents = HVSquash(
        VBox(
          TextEntry(
            Id(:ldapServer),
            _(
              "The &Host or IP address of the master server or read/write replica in the tree:"
            ),
            ldap_server
          ),
          Left(
            HVSquash(
              IntField(
                Id(:ldapPort),
                _("Secure LDAP port (e.g. 636):"),
                0,
                65535,
                ldap_port
              )
            )
          ),
          VSpacing(1.0),
          TextEntry(
            Id(:adminName),
            _("Admin &name and context (e.g. cn=admin,o=microfocus):"),
            admin_name
          ),
          Left(
            Password(
              Id(:admin_password),
              _("Enter the admin &password:"),
              admin_password
            )
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "ldap_for_schema", ""),
        Label.BackButton,
        Label.NextButton
      )

      UI.SetFocus(Id(:next))

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :next
          ldap_server = Convert.to_string(
            UI.QueryWidget(Id(:ldapServer), :Value)
          )
          if !NovellValidators.ValidateDNSorIP(ldap_server)
            Popup.Error(_("A valid IP or DNS name must be entered."))
            next
          end
          Ops.set(
            NovellSchematool.Settings,
            "CONFIG_SCHEMATOOL_LDAP_SERVER",
            ldap_server
          )

          admin_name = Convert.to_string(UI.QueryWidget(Id(:adminName), :Value))
          if !NovellValidators.ValidateFDN(admin_name, false)
            name = admin_name
            name = NovellUtils.Strip(name, " ")
            if name == nil || name == "" || name == " "
              Popup.Error(_("A valid LDAP Admin user must be entered."))
            end
            next
          end
          Ops.set(
            NovellSchematool.Settings,
            "CONFIG_SCHEMATOOL_LDAP_ADMIN",
            admin_name
          )

          admin_password = Convert.to_string(
            UI.QueryWidget(Id(:admin_password), :Value)
          )
          next if !NovellValidators.ValidatePassword(admin_password)
          Ops.set(
            NovellSchematool.Settings,
            "CONFIG_SCHEMATOOL_LDAP_ADMIN_PASSWORD",
            admin_password
          )

          ldap_port = Convert.to_integer(UI.QueryWidget(Id(:ldapPort), :Value))
          Ops.set(
            NovellSchematool.Settings,
            "CONFIG_SCHEMATOOL_LDAP_SECURE_PORT",
            ldap_port
          )

          iVal = NovellValidators.ValidateCredentials(
            admin_name,
            admin_password,
            ldap_server,
            ldap_port,
            true
          )
          Builtins.y2milestone(
            "NovellSchematool:validating creds. name %1, pass ****, ip %2, port %3",
            admin_name,
            ldap_server,
            ldap_port
          )
          if iVal != 0
            Builtins.y2milestone(
              "NovellSchematool:ValidateCredentials() returned %1",
              iVal
            )
            if iVal == -666
              err = _(
                "The attempt to perform a secure LDAP bind did not respond in 2 minutes\n" +
                  "This may indicate a problem with LDAP on the target server.\n" +
                  "If you are confident that your credentials are correct you may continue,\n" +
                  "otherwise please determine if LDAP is running correctly on the target system\n" +
                  "Do you want to continue?"
              )
              yes = Popup.YesNoHeadline(_("LDAP secure bind timeout"), err)
              next if !yes
            else
              err = Builtins.sformat(
                _(
                  "User credentials failed to validate Using:\n" +
                    "      Server: %1\n" +
                    "      User: %2\n" +
                    "      LDAP port: %3\n" +
                    "Do you want to continue anyway?"
                ),
                ldap_server,
                admin_name,
                ldap_port
              )
              yes = Popup.YesNoHeadline(_("Credentials Failed"), err)
              next if !yes
            end
          end
          break
        elsif ret == :back
          break
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end

      deep_copy(ret)
    end


    # Get the name of the service for which schema is to be extended.
    #
    # @return [String]
    def GetServiceDisplayName(service)
      lst = Builtins.splitstring(service, "=")
      return "" if lst == nil # avoid nil value
      Ops.get(lst, 0, "")
    end

    # Query UI (MultiSelection boxes) for all selected items (services)
    #
    # @return [Array<String>] of selected patterns
    def GetSelectedItems
      # Generate a list of all selected patterns (services)
      selected_services = Convert.convert(
        UI.QueryWidget(Id(:services), :SelectedItems),
        :from => "any",
        :to   => "list <integer>"
      )
      selected_services = [] if selected_services == nil # avoid nil value
      selected_schema_services = []
      pos = 0
      Builtins.foreach(selected_services) do |service|
        selected_schema_services = Builtins.add(
          selected_schema_services,
          Ops.get(@services_list, service, "")
        )
      end
      Builtins.y2milestone("selected_services: %1", selected_schema_services)
      deep_copy(selected_schema_services)
    end

    # schematool dialog 2
    # @return dialog result
    def SchemaDialog
      # Schema Tool options dialog caption
      caption = _("OES eDirectory Schema Extension Utility")

      @services_list = Ops.get_list(
        NovellSchematool.Settings,
        "CONFIG_SCHEMA_DATA_FILE",
        []
      )
      @services_list = Builtins.filter(@services_list) do |src|
        src != nil && src != ""
      end
      # Generate the widgets for the services list to display on the UI
      services_display_list = []
      pos = 0
      Builtins.foreach(@services_list) do |service|
        element = Item(
          Id(pos), # Select the checkbox if the pattern in the checked_items list
          GetServiceDisplayName(service), # name of the Service to display on the UI
          Builtins.contains(@checked_items, service)
        )
        services_display_list = Builtins.add(services_display_list, element)
        pos = Ops.add(pos, 1)
      end


      # Schema Tool dialog contents
      contents = MarginBox(
        3.5,
        0.2,
        VBox(
          Left(Label(_("Select the services to extend the schema for:"))),
          MultiSelectionBox(
            Id(:services),
            Opt(:vstretch),
            "",
            services_display_list
          ),
          VSpacing(1.0),
          HBox(
            PushButton(Id(:allBttn), Opt(:default, :key_F3), _("Sele&ct All")),
            PushButton(Id(:noneBttn), Opt(:key_F4), _("Deselec&t All"))
          )
        )
      )

      Wizard.SetContentsButtons(
        caption,
        contents,
        Ops.get_string(@HELPS, "schema_extension", ""),
        Label.BackButton,
        Label.NextButton
      )

      ret = nil
      while true
        ret = UI.UserInput

        # abort?
        if ret == :abort || ret == :cancel
          if ReallyAbort()
            break
          else
            next
          end
        elsif ret == :next
          selected = GetSelectedItems()
          # Display popup warning if nothing is selected...
          if Ops.less_or_equal(Builtins.size(selected), 0)
            Popup.Warning(_("Nothing is selected."))
            next
          end
          Ops.set(
            NovellSchematool.Settings,
            "CONFIG_SCHEMA_SELECTED_LIST",
            selected
          )
          break
        elsif ret == :allBttn
          set_list = []
          pos2 = 0
          Builtins.foreach(@services_list) do |entry|
            set_list = Builtins.add(set_list, pos2)
            pos2 = Ops.add(pos2, 1)
          end
          UI.ChangeWidget(Id(:services), :SelectedItems, set_list)
        elsif ret == :noneBttn
          UI.ChangeWidget(Id(:services), :SelectedItems, [])
        elsif ret == :back
          break
        else
          Builtins.y2error("unexpected retcode: %1", ret)
          next
        end
      end

      deep_copy(ret)
    end
  end
end
